@import "../utils";

.rio-list-view {
    pointer-events: none;

    display: flex;
    flex-direction: column;
    align-items: stretch;
    align-content: stretch;
}

.rio-heading-list-item {
    pointer-events: auto;
    box-sizing: border-box;
    margin-left: var(--rio-global-corner-radius-medium);
    margin-top: 1rem;
    margin-right: var(--rio-global-corner-radius-medium);
    margin-bottom: 0.4rem;
}

.rio-separator-list-item {
    pointer-events: none;
}

.rio-separator-list-item::after {
    content: "";
    display: block;
    position: relative;
    height: 1rem;
}

.rio-custom-list-item {
    pointer-events: auto;

    transition: background-color 0.1s ease-out;

    @include single-container();
}

.rio-custom-list-item:hover {
    background: var(--hover-color);
}

.rio-custom-list-item > * {
    padding: 0.5rem 1rem;
}

.rio-listview-grouped {
    position: relative;
    pointer-events: auto;

    background-color: var(--rio-local-bg-variant);

    transition: background-color 0.1s ease-out;
}

.rio-listview-grouped + .rio-listview-grouped::after {
    content: "";

    position: absolute;
    top: 0;
    left: 0;
    right: 0;
    height: 1px;

    background-color: var(--rio-local-text-color);
    opacity: 0.2;
}

.rio-list-item-ripple:hover {
    background: var(--rio-local-bg-active);
}

.rio-list-view.selectable .rio-custom-list-item.selected {
    background-color: var(--rio-global-secondary-bg);
}

.rio-list-view.selectable .rio-custom-list-item.selected:hover {
    background: var(--rio-global-secondary-bg-active);
}

.rio-list-view.selectable .rio-custom-list-item:hover {
    background: var(--rio-local-bg-active);
}

.rio-list-view.selectable .rio-custom-list-item {
    cursor: pointer;
}
