.rio-text {
    pointer-events: auto;

    display: flex;
    align-items: center;

    color: var(--rio-local-text-color);
}

.rio-text > * {
    flex-grow: 1;

    // Remove default styling of headings
    margin: 0;
    padding: 0;
}
