@import "../utils";

.rio-key-event-listener {
    // Enable mouse events so that users can click into the KeyEventListener to
    // give it keyboard focus
    pointer-events: auto;

    @include single-container();
}
